{% ckan_extends %}

{% block flash %}
{{ super() }}
{% asset 'ckanext-twitter/confirm-tweet' %}
{% if pkg and h.tweet_ready(pkg.id) %}
<input type="hidden" data-module="confirm-tweet" data-module-tweet="{{ h.get_tweet(pkg.id) }}"
       data-module-pkgid="{{ pkg.id }}" data-module-disable_edit="{{ h.disable_edit() }}">
{% endif %}
{% endblock %}
